import unittest
import os

from returns.result import Success

from models.Car import CarFull, CarDate
from scrapers.details.mollerAuto import scrape_moller_car_detail

THIS_FOLDER = os.path.dirname(os.path.abspath(__file__))


class UtilsTestCase(unittest.TestCase):
    def test_audi(self) -> None:
        with open(os.path.join(THIS_FOLDER, "moller_data/moller_audi.html")) as car:
            self.assertEqual(Success(CarFull(
                url="https://lietotiauto.mollerauto.lv/lv/vehicle/10236564/audi-a4-20-tfsi-110kw-aut-20-110kw-aut/",
                previewImgSrc='https://lietotiauto.mollerauto.lv/lv/vehicle/simage/10237323/10237323.jpg',
                summary='Audi A4 2.0 TFSI 110kw aut. 2.0 110kW aut',
                date=CarDate("07", "2020"),
                type='sedans',
                transmission='automātiskā',
                hp='150 ZS (110 kW)',
                price=33600,
                vin='WAUZZZF43LA059292',
                registrationNo='MN6786',
                mileage=20910,
                engineSize=2000,
                techInspDate=CarDate("07", "2022"),
                fuelType="petrol",
                body="sedan",
                drivetrain=None,
                color='melna',
                hasWarranty=True,
                doors='4/5',
                country="lv",
                dealer="moller-auto"
            )), scrape_moller_car_detail(car.read()))

    def test_audi_a6_sport(self) -> None:
        with open(os.path.join(THIS_FOLDER, "moller_data/moller_audi_a6_sport.html")) as car:
            self.assertEqual(Success(CarFull(
                url="https://lietotiauto.mollerauto.lv/lv/vehicle/10236087/audi-a6-sport-20-150kw-aut/",
                previewImgSrc='https://lietotiauto.mollerauto.lv/lv/vehicle/simage/10236092/10236092.jpg',
                summary='Audi A6 Sport 2.0 150kW aut',
                date=CarDate("05", "2021"),
                type='universāls',
                transmission='automātiskā',
                hp='204 ZS (150 kW)',
                price=62600,
                vin='WAUZZZF22MN086735',
                registrationNo='086735',
                mileage=9019,
                engineSize=2000,
                techInspDate=CarDate("05", "2024"),
                fuelType="diesel",
                body="wagon",
                drivetrain="awd",
                color='cita',
                hasWarranty=True,
                doors='4/5',
                country="lt",
                dealer="moller-auto"
            )), scrape_moller_car_detail(car.read()))

    def test_vw_amarok(self) -> None:
        with open(os.path.join(THIS_FOLDER, "moller_data/moller_vw_amarok.html")) as car:
            self.assertEqual(Success(CarFull(
                url="https://lietotiauto.mollerauto.lv/lv/vehicle/10235196/volkswagen-amarok-20-106kw-meh/",
                previewImgSrc='https://lietotiauto.mollerauto.lv/lv/vehicle/simage/10236919/10236919.jpg',
                summary='Volkswagen Amarok 2.0 106kW meh',
                date=CarDate("12", "2010"),
                type='pikaps',
                transmission='mehāniskā',
                hp='144 ZS (106 kW)',
                price=16490,
                vin='WV1ZZZ2HZB8024645',
                registrationNo='847KGR',
                mileage=133710,
                engineSize=2000,
                techInspDate=CarDate("12", "2021"),
                fuelType="diesel",
                body="pickup",
                drivetrain="awd",
                color='pelēka',
                hasWarranty=None,
                doors='4/5',
                country="ee",
                dealer="moller-auto"
            )), scrape_moller_car_detail(car.read()))

if __name__ == '__main__':
    unittest.main()



import re
from typing import Tuple, Dict, Optional

from gazpacho import Soup
from returns.pipeline import flow
from returns.result import safe

from models.Car import CarDate, CarFull, Country, FuelType, BodyType, Drivetrain
from scrapers.utils import parse_price, find_one, find_many

DOMAIN = "https://lietotiauto.mollerauto.lv"


@safe
def scrape_moller_car_detail(html: str) -> CarFull:
    soup = Soup(html)

    def get_row_tuple(row: Soup) -> Tuple[str, str]:
        def extract_text(s: Soup) -> str:
            return s.text

        name: str = flow(find_one("th")(row), lambda _: _.bind(extract_text))
        value: str = flow(find_one("td")(row), lambda _: _.bind(extract_text))
        return name, value

    def get_table_dict(which_table: int) -> Dict[str, str]:
        return flow(
            soup,
            find_many("table"),
            lambda _: _.bind(lambda v: v[which_table]),
            find_many("tr"),
            lambda _: _.bind(lambda rows: tuple(get_row_tuple(row) for row in rows)),
            dict
        )

    info = get_table_dict(0)
    details = get_table_dict(1)
    imageSrc = flow(find_one("a", {"class": "fancybox"})(soup), lambda _: _.bind(lambda v: v.attrs["href"]))
    url = flow(
        find_one("meta", {"property": "og:url"})(soup),
        lambda _: _.map(lambda v: v.attrs["content"])
    ).value_or("")

    return CarFull(
        url=url,
        summary=find_one("h1")(soup).map(lambda _: _.text).value_or(""),
        transmission=details.get('Pārnesumkārba:'),
        hp=details.get('Jauda:'),
        type=details.get('Virsbūves tips:'),
        body=parse_body(details.get('Virsbūves tips:')),
        date=parse_date(details.get('Pirmā reģistrācija:')),
        color=details.get('Krāsa:'),
        engineSize=parse_int(details.get('Dzinēja tilpums:')),
        mileage=parse_int(details.get('Nobraukums:')),
        fuelType=parse_fuel_type(details.get('Degviela:')),
        hasWarranty=parse_warranty(details.get('Garantija:')),
        doors=details.get('Durvju skaits:'),
        vin=details.get('Virsbūves (VIN) numurs:'),
        registrationNo=details.get('Reģistrācijas numurs:'),
        techInspDate=parse_date(details.get('Tehniskā skate:')),
        drivetrain=parse_drivetrain(details.get('Piedziņa:')),
        price=parse_price(info['Cena:']),
        dealer="moller-auto",
        country=parse_country(info.get('Valsts:')),
        previewImgSrc="https://lietotiauto.mollerauto.lv" + imageSrc
    )


def parse_country(country: Optional[str]) -> Optional[Country]:
    if country == "Latvija":
        return "lv"
    if country == "Lietuva":
        return "lt"
    if country == "Igaunija":
        return "ee"
    return None


def parse_date(date: Optional[str]) -> Optional[CarDate]:
    if date is None:
        return None

    d = date.split("/")
    return CarDate(d[0], d[1])


def parse_int(mileage: Optional[str]) -> Optional[int]:
    if mileage is None:
        return None

    return int(re.findall(r'\d+', mileage.replace(" ", ""))[0])


def parse_fuel_type(fuelType: Optional[str]) -> Optional[FuelType]:
    if fuelType == "benzīns":
        return "petrol"
    if fuelType == "dīzelis":
        return "diesel"
    return None


def parse_body(body: Optional[str]) -> Optional[BodyType]:
    if body == "sedans":
        return "sedan"
    if body == "universāls":
        return "wagon"
    if body == "pikaps":
        return "pickup"
    return None


def parse_warranty(warranty: Optional[str]) -> Optional[bool]:
    if warranty == "Ražotāja garantija":
        return True
    return None


def parse_drivetrain(drivetrain: Optional[str]) -> Optional[Drivetrain]:
    if drivetrain == "pilnpiedziņa":
        return "awd"
    if drivetrain == "priekšējā piedziņa":
        return "fwd"
    if drivetrain == "aizmugures piedziņa":
        return "rwd"
    return None



from dataclasses import dataclass
from typing import NamedTuple, Literal, Optional

CarDate = NamedTuple("CarDate", [("month", str), ("year", str)])
Drivetrain = Literal["fwd", "awd", "rwd"]
BodyType = Literal["sedan", "hatchback", "wagon", "pickup", "suv", "minivan", "motorcycle", "coupe"]
FuelType = Literal["petrol", "diesel", "hybrid"]
Country = Literal["lv", "lt", "ee"]
Dealer = Literal["moller-auto", "inchcape", "brc-auto"]


@dataclass
class CarFull():
    url: str
    previewImgSrc: str
    summary: str
    # first registration date
    date: Optional[CarDate]
    type: Optional[str]
    # TODO: put literal here with automatic/manual
    transmission: Optional[str]
    # TODO: put int here
    hp: Optional[str]
    price: int
    vin: Optional[str]
    registrationNo: Optional[str]
    mileage: Optional[int]
    engineSize: Optional[int]
    techInspDate: Optional[CarDate]
    fuelType: Optional[FuelType]
    body: Optional[BodyType]
    drivetrain: Optional[Drivetrain]
    color: Optional[str]
    hasWarranty: Optional[bool]
    doors: Optional[str]
    country: Optional[Country]
    dealer: Dealer



import re
from typing import Tuple, Optional

from gazpacho import Soup
from returns.pipeline import flow
from returns.pointfree import bind
from returns.result import safe

from scrapers.utils import find_one, find_many

from models.Car import CarDate, CarFull, FuelType, BodyType, Drivetrain

@safe
def scrape_inchcape_car_detail(html: str) -> CarFull:
    soup = Soup(html)

    url = flow(
        soup,
        find_one("link", {"rel": "canonical"}),
        lambda _: _.bind(lambda _: _.attrs["href"].replace("lv//", "lv/")),
    )

    previewImg = flow(
        soup,
        find_one("div", {"class": "product-card__image-wrap"}),
        lambda _: _.bind(lambda _: _.attrs["data-src"]),
    )

    summary = flow(
        find_one("h1", {"class": "product-card__title title-2"})(soup),
        lambda _: _.bind(lambda _: find_one("span")(_)),
        lambda _: _.bind(lambda _: _.text)
    )

    def get_feature(feature: Soup) -> Tuple[str, str]:
        name: str = flow(find_one("div", {"class": "product-card__feature-label"})(feature),
                         lambda _: _.bind(lambda _: _.text))
        value: str = flow(find_one("div", {"class": "product-card__feature-value"})(feature),
                          lambda _: _.bind(lambda _: _.text))

        return name, value

    features = flow(
        find_many("div", {"class": "product-card__feature"})(soup),
        lambda _: _.map(lambda fs: [get_feature(feature) for feature in fs]),
        lambda _: _.bind(lambda _: dict(_))
    )

    date = features["Izlaiduma gads"].split("-")

    price = flow(
        find_one("div", {"class": "product-card__price"})(soup),
        bind(find_one("span", {"class": "new"})),
        lambda _: _.bind(lambda _: _.text),
        parse_int
    )

    def get_info(feature: Soup) -> Tuple[str, str]:
        name: str = flow(find_one("div", {"class": "product-info__option-label"})(feature),
                         lambda _: _.bind(lambda _: _.text))
        value: str = flow(find_one("div", {"class": "product-info__option-value"})(feature),
                          lambda _: _.bind(lambda _: _.text))

        return name, value

    info = flow(
        find_many("div", {"class": "product-info__option"})(soup),
        lambda _: _.map(lambda _: [get_info(information) for information in _]),
        lambda _: _.bind(lambda _: dict(_))
    )

    return CarFull(
        url=url,
        previewImgSrc="https://certified.inchcape.lv" + previewImg,
        summary=summary,
        date=CarDate(date[1], date[0]),
        type=features.get("Virsbūves tips"),
        transmission=features.get("Pārnesumkārba"),
        hp=re.findall("[0-9]+\skW|[0-9]+\shp", summary)[0],
        price=price or -1,
        vin=info.get("VIN"),
        registrationNo=None,
        mileage=parse_int(features.get("Nobraukums")),
        engineSize=None,
        techInspDate=None,
        fuelType=parse_fuel_type(features.get("Dzinējs")),
        body=parse_body(features.get("Virsbūves tips")),
        drivetrain=parse_drivetrain(features.get("Piedziņas veids")),
        color=None,
        # Claims to have on every car for 6 months
        hasWarranty=True,
        doors=None,
        country="lv",
        dealer="inchcape",
    )


def parse_int(mileage: Optional[str]) -> Optional[int]:
    if mileage is None:
        return None

    return int(re.findall(r'\d+', mileage.replace(" ", ""))[0])


def parse_fuel_type(fuelType: Optional[str]) -> Optional[FuelType]:
    if fuelType == "Benzīns":
        return "petrol"
    if fuelType == "Dīzeļdegviela":
        return "diesel"
    if fuelType == "Plug-in Hibrīds":
        return "hybrid"
    return None


def parse_body(body: Optional[str]) -> Optional[BodyType]:
    if body == "Sedans":
        return "sedan"
    if body == "Universāls":
        return "wagon"
    if body == "Pikaps":
        return "pickup"
    if body == "SUV":
        return "suv"
    if body == "Hečbeks":
        return "hatchback"
    if body == "Minivans":
        return "minivan"
    if body == "MOTO":
        return "motorcycle"
    return None


def parse_drivetrain(drivetrain: Optional[str]) -> Optional[Drivetrain]:
    if drivetrain == "AWD":
        return "awd"
    if drivetrain == "FWD":
        return "fwd"
    if drivetrain == "RWD":
        return "rwd"
    return None



from typing import Tuple, Optional
import re
from gazpacho import Soup
from returns.pipeline import flow
from returns.result import safe

from scrapers.utils import find_one, find_many, parse_price

from models.Car import CarDate, CarFull, Drivetrain, BodyType, FuelType


@safe
def scrape_brc_auto_car_detail(html: str) -> CarFull:
    soup = Soup(html)
    summary = flow(
        soup,
        find_one("h1", {"class": "font-bold text-2xl lg:text-3xl mb-1"}),
        lambda _: _.bind(lambda _: _.text)
    )

    url = flow(
        soup,
        find_one("meta", {"property": "og:url"}),
        lambda _: _.bind(lambda _: _.attrs["content"])
    )

    previewImgSrc = flow(
        soup,
        find_one("a", {"class": "swiper-slide"}),
        lambda _: _.bind(lambda _: _.attrs["data-thumb"])
    )

    def get_spec(spec: Soup) -> Tuple[str, str]:
        name: str = flow(find_one("use")(spec), lambda _: _.bind(lambda _: _.attrs["xlink:href"]))
        value: str = flow(find_one("span")(spec), lambda _: _.bind(lambda _: _.text))
        return name, value

    data = flow(
        soup,
        find_one("ul", {"class": "car-specs"}),
        lambda _: _.bind(find_many("li")),
        lambda _: _.bind(lambda _: [get_spec(spec) for spec in _]),
        dict
    )

    vin = flow(
        soup,
        find_one("div", {"class": "car-section__content"}),
        lambda _: _.bind(find_one("p")),
        lambda _: _.bind(find_one("a")),
        lambda _: _.bind(lambda _: _.attrs["href"]),
        lambda _: _.split("=")[1]
    )

    price = flow(
        soup,
        find_one("div", {"class": "mt-1 cars-price font-black lg:text-2xl leading-none"}),
        lambda _: _.bind(lambda _: _.text)
    )

    return CarFull(
        url=url,
        previewImgSrc=previewImgSrc,
        summary=summary,
        date=CarDate(month='00', year=data["#calendar"].replace(" g.", "")),
        type=data.get("#body"),
        transmission=data.get("#gearbox"),
        hp=data.get("#power"),
        price=parse_price(price),
        vin=vin,
        registrationNo=None,
        mileage=parse_int(data.get("#speedometer")),
        engineSize=None,
        techInspDate=None,
        fuelType=parse_fuel_type(data.get("#fuel")),
        body=parse_body(data.get("#body")),
        drivetrain=parse_drivetrain(data.get("#wheels")),
        color=None,
        hasWarranty=True if "Garantija" in list(data.values()) else False,
        doors=data.get("#doors"),
        country='lv',
        dealer='brc-auto'
    )


def parse_int(mileage: Optional[str]) -> Optional[int]:
    if mileage is None:
        return None

    return int(re.findall(r'\d+', mileage.replace(" ", ""))[0])


def parse_fuel_type(fuelType: Optional[str]) -> Optional[FuelType]:
    if fuelType is None:
        return None

    if "Benzīns" in fuelType:
        return "petrol"
    if "Dīzelis" in fuelType:
        return "diesel"
    if fuelType == "Plug-in Hibrīds":
        return "hybrid"
    return None


def parse_body(body: Optional[str]) -> Optional[BodyType]:
    if body == "Sedans":
        return "sedan"
    if body == "Universālis":
        return "wagon"
    if body == "Pikaps":
        return "pickup"
    if body == "Apvidus":
        return "suv"
    if body == "Hečbeks":
        return "hatchback"
    if body == "Minivans":
        return "minivan"
    if body == "MOTO":
        return "motorcycle"
    if body == "Kupeja":
        return "coupe"
    return None


def parse_drivetrain(drivetrain: Optional[str]) -> Optional[Drivetrain]:
    if drivetrain == "Pilnpiedziņas":
        return "awd"
    if drivetrain == "Priekšējais":
        return "fwd"
    if drivetrain == "Aizmugurējais":
        return "rwd"
    return None



from typing import Optional, Dict, Any, Callable, List

from gazpacho import Soup
from returns.maybe import Maybe, Some, Nothing
import re


def find_one(tag: str, attrs: Optional[Dict[str, Any]] = None) -> Callable[[Soup], Maybe[Soup]]:
    def _(soup: Soup) -> Maybe[Soup]:
        result = soup.find(tag, attrs, partial=False)

        if type(result) is Soup:
            return Some(result)

        if type(result) is list:
            return Some(result[0])

        return Nothing

    return _


def find_many(tag: str, attrs: Optional[Dict[str, Any]] = None) -> Callable[[Soup], Maybe[List[Soup]]]:
    def _(soup: Soup) -> Maybe[List[Soup]]:
        result = soup.find(tag, attrs, partial=False, mode='all')

        if type(result) is Soup:
            return Some([result])

        if type(result) is list:
            return Some(result)

        return Nothing

    return _


def parse_price(price: str) -> int:
    return int(re.findall(r'\d+', price.replace(" ", ""))[0])



from typing import Iterable

from gazpacho import Soup
from returns.pipeline import flow
from scrapers.utils import find_one, find_many


def fetch_inchcape_urls(page: int = 1) -> Iterable[str]:
    print("fetch_inchcape_urls")
    soup = Soup.get(
        url=f"https://certified.inchcape.lv/auto-ajax?drive=AWD&catalog_page={page}&_=1637248077760",
    )

    return flow(
        soup,
        find_many("div", {"class": "offer__info-row"}),
        lambda _: _.bind(lambda _: _),
        lambda _: [flow(
            car,
            find_one("a"),
            lambda anchor: anchor.map(lambda _: _.attrs),
            lambda _: _.bind(lambda attrs: attrs["href"]),
            lambda _: f"https://certified.inchcape.lv/{_}"
        ) for car in _],
    )



import unittest
import os

from returns.result import Success

from models.Car import CarFull, CarDate
from scrapers.details.brcAuto import scrape_brc_auto_car_detail
from scrapers.details.inchcape import scrape_inchcape_car_detail

THIS_FOLDER = os.path.dirname(os.path.abspath(__file__))


class UtilsTestCase(unittest.TestCase):
    def test_mercedes_ml_500(self) -> None:
        with open(os.path.join(THIS_FOLDER, "brc_data/brc_mercedes_ml_500.html")) as car:
            self.assertEqual(Success(CarFull(
                url='https://lv.brcauto.eu/automobilis/c834749-mercedes-benz-ml-500-5.5l-automatiska',
                previewImgSrc='https://media.brcauto.eu/cars/all/834749_1.jpg',
                summary='Mercedes-Benz ML 500',
                date=CarDate(month='00', year='2007'),
                type='Apvidus',
                transmission='Automātiskā',
                hp='285 Kw (388 HP)',
                price=9000,
                vin='WDC1641721A343529',
                registrationNo=None,
                mileage=253770,
                engineSize=None,
                techInspDate=None,
                fuelType='petrol',
                body="suv",
                drivetrain='awd',
                color=None,
                hasWarranty=False,
                doors="5",
                country='lv',
                dealer='brc-auto'
            )), scrape_brc_auto_car_detail(car.read()))

    def test_ford_focus(self) -> None:
        with open(os.path.join(THIS_FOLDER, "brc_data/brc_ford_focus.html")) as car:
            self.assertEqual(Success(CarFull(
                url='https://lv.brcauto.eu/automobilis/c832614-ford-focus-1.5l-mehaniska',
                previewImgSrc='https://media.brcauto.eu/cars/all/832614_1.jpg',
                summary='Ford Focus',
                date=CarDate(month='00', year='2016'),
                type='Universālis',
                transmission='Mehāniskā',
                hp='88 Kw (120 HP)',
                price=8500,
                vin='WF06XXGCC6GC82052',
                registrationNo=None,
                mileage=143343,
                engineSize=None,
                techInspDate=None,
                fuelType='diesel',
                body="wagon",
                drivetrain='fwd',
                color=None,
                hasWarranty=False,
                doors="5",
                country='lv',
                dealer='brc-auto'
            )), scrape_brc_auto_car_detail(car.read()))

    def test_bmw_220(self) -> None:
        with open(os.path.join(THIS_FOLDER, "brc_data/brc_bmw_220.html")) as car:
            self.assertEqual(Success(CarFull(
                url='https://lv.brcauto.eu/automobilis/c832466-bmw-220-2.0l-mehaniska',
                previewImgSrc='https://media.brcauto.eu/cars/all/832466_1.jpg',
                summary='BMW 220',
                date=CarDate(month='00', year='2015'),
                type='Kupeja',
                transmission='Mehāniskā',
                hp='140 Kw (190 HP)',
                price=12900,
                vin='WBA2G71090VZ11343',
                registrationNo=None,
                mileage=219814,
                engineSize=None,
                techInspDate=None,
                fuelType='diesel',
                body="coupe",
                drivetrain='rwd',
                color=None,
                hasWarranty=True,
                doors="2",
                country='lv',
                dealer='brc-auto'
            )), scrape_brc_auto_car_detail(car.read()))


if __name__ == '__main__':
    unittest.main()



from typing import Iterable

from gazpacho import Soup
from returns.pipeline import flow
from scrapers.utils import find_one, find_many


def fetch_brc_auto_urls(page: int = 1) -> Iterable[str]:
    print("fetch_brc_auto_urls")
    soup = Soup.get(
        url=f"https://lv.brcauto.eu/lietoti-auto?city=5&search=1&driving_wheelbase=3&page={page}",
    )

    return flow(
        soup,
        find_many("div", {"class": "cars"}),
        lambda _: _.bind(lambda _: _),
        lambda _: [flow(
            car,
            find_one("h2", {"class": "cars__title"}),
            lambda _: _.bind(find_one("a")),
            lambda _: _.bind(lambda _: _.attrs["href"]),
        ) for car in _],
    )
